package midlets;


import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.TextField;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Choice;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

import peer2me.framework.*;
import peer2me.util.Log;


/**
 * 
 * This class contains a MIDlet using the Peer2Me framework v2.0. 
 * It has a simple GUI and supports sending of both text and files.
 *
 * @author Torbjrn Vatn & Steinar A. Hestnes
 */

public class Peer2MeDemoMIDlet extends MIDlet implements FrameworkListener{
		
	// A Log instance
	Log log = Log.getInstance();
	
	// The Main GUI of the MIDlet
	private MainGui mainGui;
	
	// The Log GUI of the MIDlet
	private LogGui logGui;
	
	// The Connect GUI of the MIDlet
	private ConnectGui connectGui;
	
	// The Send GUI of the MIDlet
	private SendGui sendGui;
	
	// The Send GUI of the MIDlet
	private FileGui fileGui;
    
    // A reference to the last displayed gui. Is used to go back.
    private Displayable lastGui;
	
	// The Framework instance
	Framework framework;
	
	// The variables retrived from the input fields of the GUI
	private String nodeName;

	// The preferred network of the MIDlet
	private final String preferredNetwork = "peer2me.network.bluetooth.BluetoothNetwork";
	
	// The name of the MIDlet
	private String midletName;
	
    // A list containing the addresses to the nodes added in the ConnectGui choicegroup
    private Hashtable nodeAddressList;    
    
    // A hashtable containing the addresses and names to each connected node. The names is the keys, and the values is the addresses
    private Hashtable participatingNodeNames;
	

	/**
	 * 
	 * Constructor
	 */
	public Peer2MeDemoMIDlet(){}

	protected void startApp() throws MIDletStateChangeException{
		mainGui = new MainGui();
		logGui = new LogGui();
		connectGui = new ConnectGui();
		sendGui = new SendGui();
		fileGui = new FileGui();
		// Sets the main gui as the current Displayable
		showGui(mainGui);
		// Gets an instance of the Framework
		framework = FrameworkFrontEnd.getInstance(this);
		// Setting the name of the MIDlet
		midletName = "TestMidlet";	
	}
    
	protected void pauseApp() {}
	
	/**
	 * 
	 * This method is called when the MIDLet is shut down
	 * 
	 */
	protected void destroyApp(boolean arg0) throws MIDletStateChangeException{
		// Shuts down the framework
		framework.shutdownFramework();		
		notifyDestroyed();
	}
	
	public void notifyAboutException(String location, Exception exception){}
	
	/**
	 * 
	 * This method displays the desired GUI class
	 *
	 * @param gui
	 */
	public void showGui(Displayable gui){
        // Saves a reference to the last displayed gui
        lastGui = Display.getDisplay(this).getCurrent();
		// Sets the new gui
        Display.getDisplay(this).setCurrent(gui);	
		gui.setCommandListener((CommandListener)gui);
	}
	
	
    /**
     * 
     * This method is called by the framework when a node is found. These nodes
     * are not yet connected in a network. To do this, use the 
     * Framework.connectToNodes() method.
     *
     * @param address The network adress of the node
     * @param remoteNodeName The name of the found remote name
     */
    public void notifyAboutFoundNode(String nodeAddress, String remoteNodeName){
    		connectGui.addNode(nodeAddress,remoteNodeName);
            showGui(connectGui);
	}    
    
    
    /**
     * 
     * This method is called from from the framework to notify the midlet about
     * the participating devices.
     * 
     * @param participants A hashtable that contains the names of the participants as unique keys and
     * the network addresses as values.
     *
     */
    public void notifyAboutParticipants(Hashtable participants){
        this.participatingNodeNames = participants;
        sendGui.removeParticipants();
        sendGui.addParticipants();        
        
        // Shows the gui where we can send datapackages
        if(participatingNodeNames.size()>0)showGui(sendGui);        
    }
    
    
    /**
     * 
     * This method is called from the framework whenever a text package is 
     * received from a remote node.
     *
     * @param senderName The name of the sender
     * @param textMessage The received text message.
     */
    public void notifyAboutReceivedTextPackage(String senderName, String textMessage){
    	String message = senderName+" sent : \n"+textMessage.length()+" characters.\n";
        sendGui.append(message);
    }
    
    /**
     * 
     * This method is called from the framework whenever a file package is 
     * received from a remote node.
     *
     * @param senderName The name of the sender
     * @param filePath The path to the received file
     */
    public void notifyAboutReceivedFilePackage(String senderName, String fileName){
    	String message = senderName+" sent : \n"+fileName+"\n";
        sendGui.append(message);
    }
	
	/**
	 * 
	 * This class is a private GUI class used to display GUI elements for this
	 * MIDlet
	 *
	 * @author Torbjrn Vatn & Steinar A. Hestnes
	 */
	private class MainGui extends Form implements CommandListener{
		
		
		// The OK Command
		private Command ok;
		private Command displayLog;
		private Command search;
		private Command exit;
        
		
		// The text filed use to input a name
		TextField text;
		
		/**
		 * 
		 * Constructor Extends Form to be able to display different GUI elements
		 *
		 */
		public MainGui(){
			super(midletName);
			ok = new Command("Ok", Command.OK, 0);
    		displayLog = new Command("View Log", Command.OK, 4); 
    		search = new Command("Search", Command.OK, 1);
			exit = new Command("Exit", Command.EXIT,0);	
			text = new TextField("Name", "Per Tome", 30, TextField.ANY);
		
			append(text);
			
			addCommand(ok);
            addCommand(displayLog);
            addCommand(exit);
		}
        
		
		/**
		 * This method is called when the CommandListener detects the use of a Command
		 * 
		 * @param command The command used
		 * @param disp The displayable
		 * 
		 */
		public void commandAction(Command command, Displayable disp){
			
			if(command == ok){
				// Fetches the input from the gui
				nodeName = text.getString();
				deleteAll();
                	removeCommand(ok);
                	append("Your name is "+nodeName+". \n\nPress Search if you want to discover other devices.\n\n");
				addCommand(search);

                try{                    
                    framework.initFramework(nodeName, midletName, preferredNetwork);
                }catch(Exception e){
                    append("Error initiating the framework. Please try again." +e);
                }             

            
            	}else if(command == search){
			    try{         
                    framework.startNodeSearch();
                    append("Searching for devices running Peer2MeDemoMIDlet\n");
                }catch(Exception e){
                    append("Could not start a search for other devices. Please try again."+e);
                }
                
			}else if(command == displayLog){
                logGui.append("Exception log:\n");
                logGui.append(log.getLog(Log.EXCEPTION_LOG));
                logGui.append("\n***************\n");
               
                logGui.append("Data package log:\n");
                logGui.append(log.getLog(Log.DATA_PACKAGE_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Connection log:\n");
                logGui.append(log.getLog(Log.CONNECTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Debug log:\n");
                logGui.append(log.getLog(Log.DEBUG_LOG));
                logGui.append("\n***************\n");
                showGui(logGui);
            }
			
			else if(command == exit){
				try{
					destroyApp(true);
				}catch (MIDletStateChangeException msce) {
					// This exception is ignored because the unconditional attribute of the
					// destroyApp() method is true.
				}
			}
           
		}
		
	}
	
	/**
	 * 
	 * This class is a private GUI class to display Log elements for this MIDlet
	 *
	 * @author Torbjrn Vatn & Steinar A. Hestnes
	 */
	private class LogGui extends Form implements CommandListener{
		
		// The log menu Commands
		private Command showExceptionLog;
		private Command showConnectionLog;
		private Command showDataPackageLog;
		private Command showDebugLog;
		private Command displayLog;
		private Command hide;
		private Command exit;
		
		
		public LogGui(){
			super("Log");
			// The log menu Commands
			showExceptionLog = new Command("Exception log", Command.ITEM, 4);
			showConnectionLog = new Command("Connection log", Command.ITEM, 4);
			showDataPackageLog = new Command("Data package log", Command.ITEM, 4);
			showDebugLog = new Command("Debug log", Command.ITEM, 4);
			displayLog = new Command("Full log", Command.ITEM, 1);	
			hide = new Command("Hide log", Command.OK, 0);
			exit = new Command("Exit", Command.EXIT,0);	
			
			addCommand(showExceptionLog);
			addCommand(showDataPackageLog);
			addCommand(showConnectionLog);			
			addCommand(showDebugLog);
        	addCommand(displayLog);
			addCommand(hide);
			addCommand(exit);
		}
		
		/**
		 * This method is called when the CommandListener detects the use of a Command
		 * 
		 * @param command The command used
		 * @param disp The displayable
		 * 
		 */
		public void commandAction(Command command, Displayable disp){
			if(command == showExceptionLog){
				this.deleteAll();				
				append(log.getLog(Log.EXCEPTION_LOG));
			}
			
			else if(command == showConnectionLog){
	            	deleteAll();
				append(log.getLog(Log.CONNECTION_LOG));
			}
			
			else if(command == showDataPackageLog){
	            	deleteAll();
	            	append(log.getLog(Log.DATA_PACKAGE_LOG));
			}
			
			else if(command == showDebugLog){
	            	deleteAll();
	            	append(log.getLog(Log.DEBUG_LOG));
			}
			
			else if(command == displayLog){
                deleteAll();
                logGui.append("Exception log:\n");
                logGui.append(log.getLog(Log.EXCEPTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Data package log:\n");
                logGui.append(log.getLog(Log.DATA_PACKAGE_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Connection log:\n");
                logGui.append(log.getLog(Log.CONNECTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Debug log:\n");
                logGui.append(log.getLog(Log.DEBUG_LOG));
                logGui.append("\n***************\n");        
			}
			
			else if(command == hide){
				deleteAll();
				showGui(lastGui);
			}
			
			else if(command == exit){
				try{
					destroyApp(true);
				}catch (MIDletStateChangeException msce) {
					// This exception is ignored because the unconditional attribute of the
					// destroyApp() method is true.
				}
			}
		}
	}
	/**
	 * 
	 * This class is a private GUI class to display Connect elements for this MIDlet
	 *
	 * @author Torbjrn Vatn & Steinar A. Hestnes
	 */
	private class ConnectGui extends Form implements CommandListener{
		
		// The commands
		private Command back;
		private Command displayLog;
		private Command connect;
		private Command exit;
		
		//The ChoiceGroup of 
		private ChoiceGroup nodes;
           
		
        /**
         * 
         * Constructor
         */
		public ConnectGui(){
			super("Nodes");
			
			// The ChoiceGroup
			nodes = new ChoiceGroup("Choose the node(s) to connect to",Choice.MULTIPLE);
            
            	nodeAddressList = new Hashtable();
			
			// The Commands
			back = new Command("Back",Command.BACK,1);
			displayLog = new Command("Display log",Command.ITEM,4);
			connect = new Command("Connect",Command.ITEM,0);
			exit = new Command("Exit", Command.EXIT,0);	
			
			addCommand(back);
            addCommand(connect);            
			addCommand(displayLog);	
			addCommand(exit);
			// Adds the ChoiceGroup
			append(nodes);
			
		}
		
		/**
		 * 
		 * This method adds a node name to the ChoiceGroup
		 *
		 * @param address The address of the remote node
         * @param remoteNodeName The name of the remote node
		 */
		public void addNode(String nodeAddress, String remoteNodeName){            
            nodeAddressList.put(remoteNodeName,nodeAddress);
            nodes.append(remoteNodeName,null);
		}
		
		/**
		 * This method is called when the CommandListener detects the use of a Command
		 * 
		 * @param command The command used
		 * @param disp The displayable
		 * 
		 */
		public void commandAction(Command command, Displayable disp) {
			
			if(command == back){
				showGui(lastGui);
			}
			
            else if(command == displayLog){                
                logGui.append("Exception log:\n");
                logGui.append(log.getLog(Log.EXCEPTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Data package log:\n");
                logGui.append(log.getLog(Log.DATA_PACKAGE_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Connection log:\n");
                logGui.append(log.getLog(Log.CONNECTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Debug log:\n");
                logGui.append(log.getLog(Log.DEBUG_LOG));
                logGui.append("\n***************\n");
                showGui(logGui);
			}
			
            	else if(command == connect){
				// Fetches all the selected elements in the ChoiceGroup   
            	Vector addressVector = new Vector();
				for(int i=0; i<nodes.size(); i++){					
                    if(nodes.isSelected(i)){
                    	addressVector.addElement((String)nodeAddressList.get(nodes.getString(i)));
                   	}
				} 
				String[] addresses = new String[addressVector.size()];
				addressVector.copyInto(addresses);
				if(addresses.length == 0) append("Please chose a recipient!");
				else framework.connectToNodes(addresses);
			}
			
            	else if(command == exit){
    				try{
    					destroyApp(true);
    				}catch (MIDletStateChangeException msce) {
    					// This exception is ignored because the unconditional attribute of the
    					// destroyApp() method is true.
    				}
    			}
			 
		}		
		
	}
	/**
	 * 
	 * This class is a private GUI class to display Send elements for this MIDlet
	 *
	 * @author Torbjrn Vatn & Steinar A. Hestnes
	 */
	private class SendGui extends Form implements CommandListener{
		
		// The commands
		private Command displayLog;
		private Command sendTextPackage;
		private Command sendFilePackage;
		private Command clearScreen;
		private Command exit;
		
		// The ChoiceGroup
		private ChoiceGroup connectedNodes;
		private TextField factor;
		
		/**
		 * 
		 * Constructor
		 */
		public SendGui(){
			super("Send datapackage");
			
			// Creating  the Commands
			sendTextPackage = new Command("SendTextPackage",Command.ITEM,1);
			sendFilePackage = new Command("SendFilePackage",Command.ITEM,2);
			displayLog = new Command("View log",Command.ITEM,3);
			clearScreen = new Command("Clear screen",Command.ITEM,4);
			exit = new Command("Exit", Command.EXIT,0);	

			// The ChoiceGroup containing the connected Nodes
			connectedNodes = new ChoiceGroup("Choose recipients", Choice.MULTIPLE);
			factor = new TextField("Alphabet factor","700",5,TextField.ANY);
			
			// Adding the elements
            addCommand(sendTextPackage);
            addCommand(sendFilePackage);
			addCommand(displayLog);		
			addCommand(clearScreen);
			addCommand(exit);
			append(connectedNodes);
			append(factor);
		}
		
        /**
         * 
         * This method add the connected participants to the connected ChoiceGroup
         *
         */
        public void addParticipants(){
            Enumeration names = participatingNodeNames.keys();
            while(names.hasMoreElements()){
                connectedNodes.append((String)names.nextElement(),null);
            }            
        }
        
        /**
         * 
         * This method removes all connected participants to the connected ChoiceGroup
         *
         */
        public void removeParticipants(){
            connectedNodes.deleteAll();
        }
        
		/**
		 * This method is called when the CommandListener detects the use of a Command
		 * 
		 * @param command The command used
		 * @param disp The displayable
		 * 
		 */
		public void commandAction(Command command, Displayable disp){
			
			if(command == sendTextPackage){
                Vector recipientNodes = new Vector();
                for(int i=0; i<connectedNodes.size(); i++){
                    if(connectedNodes.isSelected(i)){
                        // Gets the address to the recipient node
                        recipientNodes.addElement((String)participatingNodeNames.get(connectedNodes.getString(i)));
                    }
                }
                String[] recipientAddresses = new String[recipientNodes.size()];                
                recipientNodes.copyInto(recipientAddresses);  
                
                String alfa = "abcdefghijklmnopqrstuvwxyz";
                String message = "";
                for(int i=0;i<Integer.parseInt(factor.getString());i++){
                    message += alfa;
                }                
                
                if(recipientAddresses.length!=0)framework.sendTextPackage(recipientAddresses,message);
                else append("Please choose a recipient!\n");
                
			}else if(command == sendFilePackage){
			
				Vector recipientNodes = new Vector();
                for(int i=0; i<connectedNodes.size(); i++){
                    if(connectedNodes.isSelected(i)){
                        // Gets the address to the recipient node
                        recipientNodes.addElement((String)participatingNodeNames.get(connectedNodes.getString(i)));
                    }
                }
                String[] recipientAddresses = new String[recipientNodes.size()];                
                recipientNodes.copyInto(recipientAddresses);  
		       
                if(recipientAddresses.length!=0){                
	                // Sets the recipents and switches GUI
	                fileGui.setRecipients(recipientAddresses);
	                fileGui.fillList();
					showGui(fileGui);
                }else append("Please choose a recipient!\n");
                
            }else if(command == displayLog){
                logGui.append("Exception log:\n");
                logGui.append(log.getLog(Log.EXCEPTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Data package log:\n");
                logGui.append(log.getLog(Log.DATA_PACKAGE_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Connection log:\n");
                logGui.append(log.getLog(Log.CONNECTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Debug log:\n");
                logGui.append(log.getLog(Log.DEBUG_LOG));
                logGui.append("\n***************\n");
                showGui(logGui);
                
            }else if(command == clearScreen){
            	sendGui.deleteAll();
            	// Creates a clean SendGui
            	sendGui = new SendGui();
            	sendGui.addParticipants();
            	// Displays it
            	showGui(sendGui);
			
			}else if(command == exit){
				try{
					destroyApp(true);
				}catch (MIDletStateChangeException msce) {
					// This exception is ignored because the unconditional attribute of the
					// destroyApp() method is true.
				}
			}
		}		
	}
	
	/**
	 * 
	 * This class is a private GUI class to display FileSend elements for this MIDlet
	 *
	 * @author Torbjrn Vatn & Steinar A. Hestnes
	 */
	private class FileGui extends List implements CommandListener{
		
		// The commands
		private Command displayLog;
		private Command sendFilePackage;
		private Command updateList;
		private Command exit;
		
		// The root of the file system on the SE K750/W800
		private String root = "c:/other/Peer2Me/";
		
		// The root of the file system on the SE Emulator
		//private String root = "peer2me/";
		
		// A Enumeration containing the files in the root directory
		Enumeration files;
		
		// The addresses of the recipients of the chosen file
		String[] recipientAddresses;
		
		/**
		 * 
		 * Constructor
		 *
		 */
		public FileGui(){
			super("Send filePackage", List.EXCLUSIVE);
			
			// Creating the elements
			displayLog = new Command("View log",Command.ITEM,4);
			sendFilePackage = new Command("Send File",Command.ITEM,2);
			updateList = new Command("Update List",Command.ITEM,2);
			exit = new Command("Exit", Command.EXIT,0);	
			 
			// Adds the commands
			addCommand(displayLog);
			addCommand(sendFilePackage);
			addCommand(updateList);
			addCommand(exit);
		}
		
		/**
		 * 
		 * This method fills the fileList List with the names of the files
		 * located in root.
		 *
		 */
		public void fillList(){
			// Fills the files Enumeration
			files = framework.getFileList(root);
			
			if(files != null){
				// Fills the filesList List
				while (files.hasMoreElements()) {
					append((String) files.nextElement(), null);					
				}
			}
		}
		
		/**
		 * 
		 * This method sets the recipientAddresses
		 * 
		 * @param recipientAddresses The recipientAddresses to set
		 */
		public void setRecipients(String[] recipientAddresses){
			this.recipientAddresses = recipientAddresses;
		}
		
		/**
		 * This method is called when the CommandListener detects the use of a Command
		 * 
		 * @param command The command used
		 * @param disp The displayable
		 * 
		 */
		public void commandAction(Command command, Displayable disp){
			
			if(command == sendFilePackage){
	            
				framework.sendFilePackage(recipientAddresses,root+getString(getSelectedIndex()));
				fileGui.deleteAll();
				showGui(sendGui);
				
			}
			
			else if(command == updateList){
				fillList();
			}
			
			
			else if(command == displayLog){
                logGui.append("Exception log:\n");
                logGui.append(log.getLog(Log.EXCEPTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Data package log:\n");
                logGui.append(log.getLog(Log.DATA_PACKAGE_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Connection log:\n");
                logGui.append(log.getLog(Log.CONNECTION_LOG));
                logGui.append("\n***************\n");
                
                logGui.append("Debug log:\n");
                logGui.append(log.getLog(Log.DEBUG_LOG));
                logGui.append("\n***************\n");
                showGui(logGui);
            }
			
			else if(command == exit){
				try{
					destroyApp(true);
				}catch (MIDletStateChangeException msce) {
					// This exception is ignored because the unconditional attribute of the
					// destroyApp() method is true.
				}
			}
			
		}
	}
}
